/*
	innoLookup - AGI lookup tool for swiss phone numbers
	
	v3.3
	
	Copyright (c) 2011 Innotronic Ingenieurbüro GmbH
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <string>
#include <iostream>

#include <libxml/parser.h>
#include <libxml/xpath.h>

#include "curl/curl.h"
#include "curlError.h"
#include "xml2Error.h"

//#include "cxxtools/iconverter.h"


using namespace std;


static char		ErrorBuffer[ CURL_ERROR_SIZE ];
static string	DataBuffer;



static int dataReader( char *data, size_t size, size_t nmemb, std::string *buf )
{
	int result = 0;
	
	if( buf != NULL )
	{
		buf->append( data, size * nmemb );
		result = size * nmemb;
	}
	
	return result;
}


bool httpLookup( const string& reqURL, const string& xpathPattern, string& callerIDname )
{
	CURL			*curl;
	CURLcode		curlResult;
	
	curl = curl_easy_init();
	
	if( ! curl )
		throw curlError( "Error: curl_easy_init() failed!" );
	
	DataBuffer.clear();
	
	curl_easy_setopt( curl, CURLOPT_ERRORBUFFER,	ErrorBuffer		);
	curl_easy_setopt( curl, CURLOPT_URL,			reqURL.c_str()	);
	curl_easy_setopt( curl, CURLOPT_HEADER,			0				);
	curl_easy_setopt( curl, CURLOPT_FOLLOWLOCATION, 1				);
	curl_easy_setopt( curl, CURLOPT_WRITEFUNCTION,	dataReader		);
	curl_easy_setopt( curl, CURLOPT_WRITEDATA,		&DataBuffer		);
	curl_easy_setopt( curl, CURLOPT_TIMEOUT,		2				);
	
	curlResult = curl_easy_perform( curl );
	curl_easy_cleanup( curl );
	
	if( curlResult == CURLE_OPERATION_TIMEDOUT )
		return( false );
	
	if( curlResult != CURLE_OK )
		throw curlError( curlResult, ErrorBuffer );
	
	
	xmlDocPtr xmlDoc = xmlParseMemory( DataBuffer.c_str(), DataBuffer.length() );
	
	if( xmlDoc == NULL )
		throw xml2Error( "Error: xmlParseMemory() failed." );
	
	xmlXPathContextPtr xpathCtx = xmlXPathNewContext( xmlDoc );
	
	if( xpathCtx == NULL )
		throw xml2Error( xmlDoc, "Error: xmlXPathNewContext() failed." );
	
	xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression( (xmlChar *)xpathPattern.c_str(), xpathCtx );
	xmlXPathFreeContext( xpathCtx );
	
	if( xpathObj == NULL )
		throw xml2Error( xmlDoc, "Error: xmlXPathEvalExpression() failed." );
	
	if( xmlXPathNodeSetIsEmpty( xpathObj->nodesetval ))
	{
		xmlXPathFreeObject( xpathObj );
		xmlFreeDoc( xmlDoc );
		return false;
	}
	
	callerIDname = (string)(char *)xmlXPathCastToString( xpathObj );
	
	xmlXPathFreeObject( xpathObj );
	xmlFreeDoc( xmlDoc );
	
	
	// cxxtools::IConverter callerIDconv( "UTF-8", "ISO8859-1" );
	// callerIDname = callerIDconv.convert( callerIDname );	
	
	return true;
}


bool lookupSearch( const string& callerIDnum, string& callerIDname )
{
	string reqURL = (string)"http://tel.search.ch/api/?was=" + callerIDnum;
	string xpathPattern = (string)"//*[local-name()='entry']/*[local-name()='title']";
	
	if( httpLookup( reqURL, xpathPattern, callerIDname ))
	{
		cout << "VERBOSE \"innoLookup: found by tel.search.ch '" << callerIDname << "'\"" << endl;
		return true;
	}

	return false;
}

bool lookupLocal( const string& callerIDnum, string& callerIDname )
{
	string reqURL = (string)"http://tel.local.ch/de/q/?ext=1&phone=" + callerIDnum;
	string xpathPattern = (string)"//*[@class = 'fn']";
	
	if( httpLookup( reqURL, xpathPattern, callerIDname ))
	{
		cout << "VERBOSE \"innoLookup: found by tel.local.ch '" << callerIDname << "'\"" << endl;
		return true;
	}

	return false;
}


int main( int argc, char **argv )
{
	string			callerIDnum, callerIDname;
	
	if( argc < 2 )
	{
		cerr << ""																			<< endl;
		cerr << "innoLookup - AGI lookup tool for swiss phone numbers"						<< endl;
		cerr <<	""																			<< endl;
		cerr <<	"v3.3"																		<< endl;
		cerr <<	""																			<< endl;
		cerr << "Usage:  innoLookup XXXXXXXXXX"												<< endl;
		cerr << "		 exten => 0XXXXXXXXX,n,AGI(innoLookup|${CALLERID(num)})"			<< endl;
		cerr << ""																			<< endl;
		cerr << "A small AGI tool that looks up the phone number given as the first"		<< endl;
		cerr << "argument at 'tel.local.ch' and 'tel.search.ch'. If a name could be "		<< endl;
		cerr << "resolved by one of the services, CALLERID is set accordingly."				<< endl;
		cerr << ""																			<< endl;
		cerr << "Any comments, bug reports or questions are always welcome."				<< endl;
		cerr << ""																			<< endl;
		cerr << "Innotronic Ingenieurbüro GmbH"												<< endl;
		cerr << "<http://www.inno.ch/innoLookup.html>"										<< endl;
		cerr << ""																			<< endl;
		cerr << "Copyright (c) 2011 Innotronic Ingenieurbüro GmbH"							<< endl;
		cerr << "Copyright (c) 1996 - 2007 Daniel Stenberg, <daniel@haxx.se>"				<< endl;
		cerr << "Copyright (c) 2003 Tommi Maekitalo"										<< endl;
		cerr << ""																			<< endl;
		
		return 0;
	}
	
	
	try
	{
		callerIDnum = (string) argv[ 1 ];
		
		if( callerIDnum.empty() || callerIDnum.length() != 10 )
		{
			cout << "VERBOSE \"innoLookup: invalid number format!\"" << endl;
			return 0;
		}
		
		if( lookupSearch( callerIDnum, callerIDname ) || lookupLocal( callerIDnum, callerIDname ))
		{
			cout << "SET CALLERID \"" << callerIDname << " <" << callerIDnum << ">\"" << endl;
		}
		else
		{
			cout << "VERBOSE \"innoLookup: unknown\"" << endl;			
		}
		
		return 0;
	}
	
	catch( const std::exception& err )
	{
		cerr << "Error: " << err.what() << endl;
	}
	
	catch( const char* err )
	{
		cerr << "Error: " << err << endl;
	}
	
	catch( ... )
	{
		cerr << "Error: unknown reason." << endl;
	}
	
	return -1;
}
